# -*- coding: utf-8 -*-

#: European Payments Council list of SEPA scheme countries as of 23 Jan 2014.
#: http://www.europeanpaymentscouncil.eu/index.cfm/knowledge-bank/epc-documents/epc-list-of-sepa-scheme-countries/
IBAN_SEPA_COUNTRIES = (
    'AT',  # Austria
    'BE',  # Belgium
    'BG',  # Bulgaria
    'ES',  # Canary Islands
    'HR',  # Croatia
    'CH',  # Switzerland
    'CY',  # Cyprus
    'CZ',  # Czech Republic
    'DE',  # Germany
    'DK',  # Denmark
    'EE',  # Estonia
    'ES',  # Spain
    'FI',  # Finland + Åland Islands
    'FR',  # France +  French Guiana, Guadeloupe, Martinique, Mayotte, Réunion, Saint Barthélemy,
           #           Saint Martin (French part), Saint Pierre and Miquelon
           #
    'GB',  # United Kingdom
    'GI',  # Gibraltar
    'GR',  # Greece
    'HU',  # Hungary
    'IE',  # Ireland
    'IS',  # Iceland
    'IT',  # Italy
    'LV',  # Latvia
    'LI',  # Liechtenstein
    'LT',  # Lithuania
    'LU',  # Luxembourg
    'MC',  # Monaco
    'MT',  # Malta
    'NL',  # Netherlands
    'NO',  # Norway
    'PL',  # Poland
    'PT',  # Portugal + Azores + Madeira
    'RO',  # Romania
    'SE',  # Sweden
    'SI',  # Slovenia
    'SK',  # Slovakia
    'SM',  # San Marino
)
